@@ -1,7 +1,7 @@
 package Perinci::Sub::Complete;
 
-our $DATE = '2015-01-09'; # DATE
-our $VERSION = '0.71'; # VERSION
+our $DATE = '2015-03-01'; # DATE
+our $VERSION = '0.72'; # VERSION
 
 use 5.010001;
 use strict;
@@ -364,6 +364,25 @@ sub complete_arg_val {
             return; # from eval
         }
 
+        my $ent = $arg_p->{'x.schema.entity'};
+        if ($ent) {
+            require Module::Path::More;
+            my $mod = "Perinci::Sub::ArgEntity::$ent";
+            if (Module::Path::More::module_path(module=>$mod)) {
+                $log->tracef("[comp][periscomp] loading module %s ...", $mod);
+                my $mod_pm = $mod; $mod_pm =~ s!::!/!g; $mod_pm .= ".pm";
+                require $mod_pm;
+                if (defined &{"$mod\::complete_arg_val"}) {
+                    $log->tracef("[comp][periscomp] invoking complete_arg_val from %s ...", $mod);
+                    my $fref = \&{"$mod\::complete_arg_val"};
+                    $fres = $fref->(
+                        %$extras,
+                        word=>$word, ci=>$ci, arg=>$arg, args=>$args{args});
+                    return; # from eval
+                }
+            }
+        }
+
         my $sch = $arg_p->{schema};
         unless ($sch) {
             $log->tracef("[comp][periscomp] arg spec does not specify schema, declining");
@@ -479,6 +498,25 @@ sub complete_arg_elem {
             return; # from eval
         }
 
+        my $ent = $arg_p->{'x.schema.element_entity'};
+        if ($ent) {
+            require Module::Path::More;
+            my $mod = "Perinci::Sub::ArgEntity::$ent";
+            if (Module::Path::More::module_path(module=>$mod)) {
+                $log->tracef("[comp][periscomp] loading module %s ...", $mod);
+                my $mod_pm = $mod; $mod_pm =~ s!::!/!g; $mod_pm .= ".pm";
+                require $mod_pm;
+                if (defined &{"$mod\::complete_arg_val"}) {
+                    $log->tracef("[comp][periscomp] invoking complete_arg_val from %s ...", $mod);
+                    my $fref = \&{"$mod\::complete_arg_val"};
+                    $fres = $fref->(
+                        %$extras,
+                        word=>$word, ci=>$ci, arg=>$arg, args=>$args{args});
+                    return; # from eval
+                }
+            }
+        }
+
         my $sch = $arg_p->{schema};
         unless ($sch) {
             $log->tracef("[comp][periscomp] arg spec does not specify schema, declining");
@@ -863,7 +901,7 @@ Perinci::Sub::Complete - Complete command-line argument using Rinci metadata
 
 =head1 VERSION
 
-This document describes version 0.71 of Perinci::Sub::Complete (from Perl distribution Perinci-Sub-Complete), released on 2015-01-09.
+This document describes version 0.72 of Perinci::Sub::Complete (from Perl distribution Perinci-Sub-Complete), released on 2015-03-01.
 
 =head1 SYNOPSIS
 
@@ -877,9 +915,8 @@ this module.
 
 =head2 complete_arg_elem(%args) -> array
 
-{en_US Given argument name and function metadata, complete array element}.
+Given argument name and function metadata, complete array element.
 
-{en_US 
 Will attempt to complete using the completion routine specified in the argument
 specification (the C<completion> property, or in the case of C<complete_arg_elem>
 function, the C<element_completion> property), or if that is not specified, from
@@ -909,7 +946,6 @@ Completion routine should return a completion answer structure (described in
 C<Complete>) which is either a hash or an array. The simplest form of answer is
 just to return an array of strings. Completion routine can also return undef to
 express declination.
-}
 
 Arguments ('*' denotes required arguments):
 
@@ -917,39 +953,36 @@ Arguments ('*' denotes required arguments):
 
 =item * B<arg>* => I<str>
 
-{en_US Argument name}.
+Argument name.
 
 =item * B<args> => I<hash>
 
-{en_US Collected arguments so far, will be passed to completion routines}.
+Collected arguments so far, will be passed to completion routines.
 
 =item * B<ci> => I<bool> (default: 0)
 
-{en_US Whether to be case-insensitive}.
+Whether to be case-insensitive.
 
 =item * B<extras> => I<hash>
 
-{en_US Add extra arguments to completion routine}.
+Add extra arguments to completion routine.
 
-{en_US 
 The keys from this C<extras> hash will be merged into the final C<%args> passed to
 completion routines. Note that standard keys like C<word>, C<cword>, C<ci>, and so
 on as described in the function description will not be overwritten by this.
-}
 
 =item * B<index> => I<int>
 
-{en_US Index of element to complete}.
+Index of element to complete.
 
 =item * B<meta>* => I<hash>
 
-{en_US Rinci function metadata, must be normalized}.
+Rinci function metadata, must be normalized.
 
 =item * B<riap_client> => I<obj>
 
-{en_US Optional, to perform complete_arg_val to the server}.
+Optional, to perform complete_arg_val to the server.
 
-{en_US 
 When the argument spec in the Rinci metadata contains C<completion> key, this
 means there is custom completion code for that argument. However, if retrieved
 from a remote server, sometimes the C<completion> key no longer contains the code
@@ -959,27 +992,22 @@ on the server.
 If supplied this argument and te C<riap_server_url> argument, the function will
 try to request to the server (via Riap request C<complete_arg_val>). Otherwise,
 the function will just give up/decline completing.
-}
 
 =item * B<riap_server_url> => I<str>
 
-{en_US Optional, to perform complete_arg_val to the server}.
+Optional, to perform complete_arg_val to the server.
 
-{en_US 
 See the C<riap_client> argument.
-}
 
 =item * B<riap_uri> => I<str>
 
-{en_US Optional, to perform complete_arg_val to the server}.
+Optional, to perform complete_arg_val to the server.
 
-{en_US 
 See the C<riap_client> argument.
-}
 
 =item * B<word> => I<str> (default: "")
 
-{en_US Word to be completed}.
+Word to be completed.
 
 =back
 
@@ -988,9 +1016,8 @@ Return value:  (array)
 
 =head2 complete_arg_val(%args) -> array
 
-{en_US Given argument name and function metadata, complete value}.
+Given argument name and function metadata, complete value.
 
-{en_US 
 Will attempt to complete using the completion routine specified in the argument
 specification (the C<completion> property, or in the case of C<complete_arg_elem>
 function, the C<element_completion> property), or if that is not specified, from
@@ -1020,7 +1047,6 @@ Completion routine should return a completion answer structure (described in
 C<Complete>) which is either a hash or an array. The simplest form of answer is
 just to return an array of strings. Completion routine can also return undef to
 express declination.
-}
 
 Arguments ('*' denotes required arguments):
 
@@ -1028,35 +1054,32 @@ Arguments ('*' denotes required arguments):
 
 =item * B<arg>* => I<str>
 
-{en_US Argument name}.
+Argument name.
 
 =item * B<args> => I<hash>
 
-{en_US Collected arguments so far, will be passed to completion routines}.
+Collected arguments so far, will be passed to completion routines.
 
 =item * B<ci> => I<bool> (default: 0)
 
-{en_US Whether to be case-insensitive}.
+Whether to be case-insensitive.
 
 =item * B<extras> => I<hash>
 
-{en_US Add extra arguments to completion routine}.
+Add extra arguments to completion routine.
 
-{en_US 
 The keys from this C<extras> hash will be merged into the final C<%args> passed to
 completion routines. Note that standard keys like C<word>, C<cword>, C<ci>, and so
 on as described in the function description will not be overwritten by this.
-}
 
 =item * B<meta>* => I<hash>
 
-{en_US Rinci function metadata, must be normalized}.
+Rinci function metadata, must be normalized.
 
 =item * B<riap_client> => I<obj>
 
-{en_US Optional, to perform complete_arg_val to the server}.
+Optional, to perform complete_arg_val to the server.
 
-{en_US 
 When the argument spec in the Rinci metadata contains C<completion> key, this
 means there is custom completion code for that argument. However, if retrieved
 from a remote server, sometimes the C<completion> key no longer contains the code
@@ -1066,27 +1089,22 @@ on the server.
 If supplied this argument and te C<riap_server_url> argument, the function will
 try to request to the server (via Riap request C<complete_arg_val>). Otherwise,
 the function will just give up/decline completing.
-}
 
 =item * B<riap_server_url> => I<str>
 
-{en_US Optional, to perform complete_arg_val to the server}.
+Optional, to perform complete_arg_val to the server.
 
-{en_US 
 See the C<riap_client> argument.
-}
 
 =item * B<riap_uri> => I<str>
 
-{en_US Optional, to perform complete_arg_val to the server}.
+Optional, to perform complete_arg_val to the server.
 
-{en_US 
 See the C<riap_client> argument.
-}
 
 =item * B<word> => I<str> (default: "")
 
-{en_US Word to be completed}.
+Word to be completed.
 
 =back
 
@@ -1095,14 +1113,12 @@ Return value:  (array)
 
 =head2 complete_cli_arg(%args) -> hash
 
-{en_US Complete command-line argument using Rinci function metadata}.
+Complete command-line argument using Rinci function metadata.
 
-{en_US 
 This routine uses C<Perinci::Sub::GetArgs::Argv> to generate C<Getopt::Long>
 specification from arguments list in Rinci function metadata and common options.
 Then, it will use C<Complete::Getopt::Long> to complete option names, option
 values, as well as arguments.
-}
 
 Arguments ('*' denotes required arguments):
 
@@ -1110,9 +1126,8 @@ Arguments ('*' denotes required arguments):
 
 =item * B<common_opts> => I<hash>
 
-{en_US Common options}.
+Common options.
 
-{en_US 
 A hash where the values are hashes containing these keys: C<getopt> (Getopt::Long
 option specification), C<handler> (Getopt::Long handler). Will be passed to
 C<get_args_from_argv()>. Example:
@@ -1130,13 +1145,10 @@ C<get_args_from_argv()>. Example:
      },
  }
 
-}
-
 =item * B<completion> => I<code>
 
-{en_US Supply custom completion routine}.
+Supply custom completion routine.
 
-{en_US 
 If supplied, instead of the default completion routine, this code will be called
 instead. Will receive all arguments that C<Complete::Getopt::Long> will pass, and
 additionally:
@@ -1148,50 +1160,44 @@ additionally:
 =item * C<args> (hash, the function arguments formed so far)
 
 =item * C<index> (int, if completing argument element value)
-}
 
 =back
 
 =item * B<cword>* => I<int>
 
-{en_US On which argument cursor is located (zero-based)}.
+On which argument cursor is located (zero-based).
 
 =item * B<extras> => I<hash>
 
-{en_US Add extra arguments to completion routine}.
+Add extra arguments to completion routine.
 
-{en_US 
 The keys from this C<extras> hash will be merged into the final C<%args> passed to
 completion routines. Note that standard keys like C<word>, C<cword>, C<ci>, and so
 on as described in the function description will not be overwritten by this.
-}
 
 =item * B<func_arg_starts_at> => I<int> (default: 0)
 
-{en_US 
 This is a (temporary?) workaround for Perinci::CmdLine. In an application with
 subcommands (e.g. C<cmd --verbose subcmd arg0 arg1 ...>), then C<words> will still
 contain the subcommand name. Positional function arguments then start at 1 not
 0. This option allows offsetting function arguments.
-}
 
 =item * B<meta>* => I<hash>
 
-{en_US Rinci function metadata}.
+Rinci function metadata.
 
 =item * B<per_arg_json> => I<bool>
 
-{en_US Will be passed to Perinci::Sub::GetArgs::Argv}.
+Will be passed to Perinci::Sub::GetArgs::Argv.
 
 =item * B<per_arg_yaml> => I<bool>
 
-{en_US Will be passed to Perinci::Sub::GetArgs::Argv}.
+Will be passed to Perinci::Sub::GetArgs::Argv.
 
 =item * B<riap_client> => I<obj>
 
-{en_US Optional, to perform complete_arg_val to the server}.
+Optional, to perform complete_arg_val to the server.
 
-{en_US 
 When the argument spec in the Rinci metadata contains C<completion> key, this
 means there is custom completion code for that argument. However, if retrieved
 from a remote server, sometimes the C<completion> key no longer contains the code
@@ -1201,49 +1207,40 @@ on the server.
 If supplied this argument and te C<riap_server_url> argument, the function will
 try to request to the server (via Riap request C<complete_arg_val>). Otherwise,
 the function will just give up/decline completing.
-}
 
 =item * B<riap_server_url> => I<str>
 
-{en_US Optional, to perform complete_arg_val to the server}.
+Optional, to perform complete_arg_val to the server.
 
-{en_US 
 See the C<riap_client> argument.
-}
 
 =item * B<riap_uri> => I<str>
 
-{en_US Optional, to perform complete_arg_val to the server}.
+Optional, to perform complete_arg_val to the server.
 
-{en_US 
 See the C<riap_client> argument.
-}
 
 =item * B<words>* => I<array[str]>
 
-{en_US Command-line arguments}.
+Command-line arguments.
 
 =back
 
 Return value:  (hash)
 
 
-{en_US 
 You can use C<format_completion> function in C<Complete::Bash> module to format
 the result of this function for bash.
-}
 
 
 =head2 complete_from_schema(%args) -> [status, msg, result, meta]
 
-{en_US Complete a value from schema}.
+Complete a value from schema.
 
-{en_US 
 Employ some heuristics to complete a value from Sah schema. For example, if
 schema is C<< [str =E<gt> in =E<gt> [qw/new open resolved rejected/]] >>, then we can
 complete from the C<in> clause. Or for something like C<< [int =E<gt> between =E<gt> [1,
 20]] >> we can complete using values from 1 to 20.
-}
 
 Arguments ('*' denotes required arguments):
 
@@ -1253,7 +1250,7 @@ Arguments ('*' denotes required arguments):
 
 =item * B<schema>* => I<any>
 
-{en_US Must be normalized}.
+Must be normalized.
 
 =item * B<word>* => I<str> (default: "")
 
@@ -1284,7 +1281,7 @@ Please visit the project's homepage at L<https://metacpan.org/release/Perinci-Su
 
 =head1 SOURCE
 
-Source repository is at L<https://github.com/perlancar/perl-Perinci-Sub-Complete>.
+Source repository is at L<https://github.com/sharyanto/perl-Perinci-Sub-Complete>.
 
 =head1 BUGS
 
